/* name-spell-checker version 0.0.4 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('nspell'), require('name-dic')) :
    typeof define === 'function' && define.amd ? define(['nspell', 'name-dic'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.NameSpellChecker = factory(global.nspell, global.nameDic));
})(this, (function (nspell, nameDic) { 'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var nspell__default = /*#__PURE__*/_interopDefaultLegacy(nspell);

    var getInnerName = (function (str) {
        return str.toLocaleLowerCase().replace(/[^A-Za-z0-9]/g, "");
    });

    var add = (function (context, name) {
        var innerName = getInnerName(name);
        var names = context.innerNameMap.get(innerName);
        if (!names) {
            names = [name];
            context.innerNameMap.set(innerName, names);
            context.nspellInstance.add(innerName);
            return;
        }
        if (names.includes(name)) {
            return;
        }
        names.push(name);
    });

    var correct = (function (context, str) {
        var innerStr = getInnerName(str);
        var names = context.innerNameMap.get(innerStr);
        if (names) {
            return names.includes(str);
        }
        return false;
    });

    var dictionary = (function (context, dic) {
        dic
            .split("\n")
            .slice(1)
            .forEach(function (name) {
            add(context, name);
        });
    });

    var BuiltInDictionary;
    (function (BuiltInDictionary) {
        BuiltInDictionary["frontEnd"] = "frontEnd";
        BuiltInDictionary["programmingLanguages"] = "programmingLanguages";
    })(BuiltInDictionary || (BuiltInDictionary = {}));

    var getDictionaries = (function (arr) {
        return arr.reduce(function (prev, cur) {
            if (typeof cur === "string") {
                switch (cur) {
                    case BuiltInDictionary.frontEnd:
                        prev.push(nameDic.frontEnd);
                        break;
                }
            }
            else {
                prev.push(cur);
            }
            return prev;
        }, []);
    });

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
            if (ar || !(i in from)) {
                if (!ar) ar = Array.prototype.slice.call(from, 0, i);
                ar[i] = from[i];
            }
        }
        return to.concat(ar || Array.prototype.slice.call(from));
    }

    var getInnerDictionaries = (function (dictionaries) {
        var innerDictionaries = [];
        dictionaries.forEach(function (dictionary) {
            var dic = dictionary.dic
                .trim()
                .split("\n")
                .map(function (name, i) {
                if (i === 0) {
                    return name;
                }
                return getInnerName(name);
            })
                .join("\n");
            innerDictionaries.push(__assign(__assign({}, dictionary), { dic: dic }));
        });
        return innerDictionaries;
    });

    function getInnerNameMap (context) {
        var map = new Map();
        context.dictionaries.forEach(function (_a, i) {
            var dic = _a.dic;
            var innerDic = context.innerDictionaries[i].dic.trim().split("\n");
            dic
                .trim()
                .split("\n")
                .forEach(function (name, i) {
                if (i === 0) {
                    return;
                }
                var innerName = innerDic[i];
                var arr = map.get(innerName);
                if (!arr) {
                    arr = [name];
                    map.set(innerName, arr);
                    return;
                }
                arr.push(name);
            });
        });
        return map;
    }

    var remove = (function (context, name) {
        var innerName = getInnerName(name);
        var names = context.innerNameMap.get(innerName);
        if (!names) {
            return;
        }
        var index = names.indexOf(name);
        if (~index) {
            names.splice(index, 1);
        }
        if (names.length === 0) {
            context.innerNameMap.delete(innerName);
            context.nspellInstance.remove(innerName);
        }
    });

    var personal = (function (context, dic) {
        dic
            .split("\n")
            .slice(1)
            .forEach(function (name) {
            if (name.startsWith("*")) {
                remove(context, name.substring(1));
                return;
            }
            add(context, name);
        });
    });

    var similarity = (function (str1, str2) {
        var seam = 0;
        seam -= str2.length - str1.length;
        return seam;
    });

    var suggest = (function (context, str) {
        var names = __spreadArray([], __read(context.innerNameMap.keys()), false);
        var pstr = getInnerName(str);
        var substringSuggests = names.filter(function (name) {
            for (var i = 0; i < name.length; i++) {
                if (name.length > pstr.length) {
                    if (name.substring(i).startsWith(pstr)) {
                        return true;
                    }
                }
                else {
                    if (pstr.substring(i).startsWith(name)) {
                        return true;
                    }
                }
                return false;
            }
            return false;
        });
        var nspellSuggests = context.nspellInstance.suggest(pstr);
        var suggests = __spreadArray([], __read(new Set(substringSuggests.concat(nspellSuggests))), false);
        return suggests
            .reduce(function (prev, cur) {
            prev = prev.concat(context.innerNameMap.get(cur) || []);
            return prev;
        }, [])
            .map(function (d) { return [similarity(pstr, d), d]; })
            .sort(function (a, b) { return b[0] - a[0]; })
            .map(function (d) { return d[1]; });
    });

    var defaultDictionaries = Object.freeze({
        frontEnd: BuiltInDictionary.frontEnd,
    });
    var NameSpellChecker = /** @class */ (function () {
        function NameSpellChecker(dictionaries) {
            if (!dictionaries) {
                this.dictionaries = getDictionaries(Object.values(NameSpellChecker.defaultDictionaries));
            }
            else {
                this.dictionaries = getDictionaries(dictionaries);
            }
            this.innerDictionaries = getInnerDictionaries(this.dictionaries);
            this.innerNameMap = getInnerNameMap(this);
            this.nspellInstance = nspell__default["default"](this.innerDictionaries);
        }
        Object.defineProperty(NameSpellChecker, "defaultDictionaries", {
            get: function () {
                return defaultDictionaries;
            },
            enumerable: false,
            configurable: true
        });
        NameSpellChecker.prototype.suggest = function (str) {
            return suggest(this, str);
        };
        NameSpellChecker.prototype.correct = function (str) {
            return correct(this, str);
        };
        NameSpellChecker.prototype.add = function (brandName) {
            add(this, brandName);
            return this;
        };
        NameSpellChecker.prototype.remove = function (brandName) {
            remove(this, brandName);
            return this;
        };
        NameSpellChecker.prototype.dictionary = function (dic) {
            dictionary(this, dic);
            return this;
        };
        NameSpellChecker.prototype.personal = function (personalDic) {
            personal(this, personalDic);
            return this;
        };
        return NameSpellChecker;
    }());

    return NameSpellChecker;

}));
