/* nspell version undefined */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.nspell = factory());
})(this, (function () { 'use strict';

  /*!
   * Determine if an object is a Buffer
   *
   * @author   Feross Aboukhadijeh <https://feross.org>
   * @license  MIT
   */

  var isBuffer = function isBuffer (obj) {
    return obj != null && obj.constructor != null &&
      typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
  };

  var ruleCodes_1 = ruleCodes;

  var NO_CODES$1 = [];

  // Parse rule codes.
  function ruleCodes(flags, value) {
    var index = 0;
    var result;

    if (!value) return NO_CODES$1

    if (flags.FLAG === 'long') {
      // Creating an array of the right length immediately
      // avoiding resizes and using memory more efficiently
      result = new Array(Math.ceil(value.length / 2));

      while (index < value.length) {
        result[index / 2] = value.slice(index, index + 2);
        index += 2;
      }

      return result
    }

    return value.split(flags.FLAG === 'num' ? ',' : '')
  }

  var parse$2 = ruleCodes_1;

  var affix_1 = affix$1;

  var push$3 = [].push;

  // Relative frequencies of letters in the English language.
  var alphabet = 'etaoinshrdlcumwfgypbvkjxqz'.split('');

  // Expressions.
  var whiteSpaceExpression$1 = /\s+/;

  // Defaults.
  var defaultKeyboardLayout = [
    'qwertzuop',
    'yxcvbnm',
    'qaw',
    'say',
    'wse',
    'dsx',
    'sy',
    'edr',
    'fdc',
    'dx',
    'rft',
    'gfv',
    'fc',
    'tgz',
    'hgb',
    'gv',
    'zhu',
    'jhn',
    'hb',
    'uji',
    'kjm',
    'jn',
    'iko',
    'lkm'
  ];

  // Parse an affix file.
  // eslint-disable-next-line complexity
  function affix$1(doc) {
    var rules = Object.create(null);
    var compoundRuleCodes = Object.create(null);
    var flags = Object.create(null);
    var replacementTable = [];
    var conversion = {in: [], out: []};
    var compoundRules = [];
    var aff = doc.toString('utf8');
    var lines = [];
    var last = 0;
    var index = aff.indexOf('\n');
    var parts;
    var line;
    var ruleType;
    var count;
    var remove;
    var add;
    var source;
    var entry;
    var position;
    var rule;
    var value;
    var offset;
    var character;

    flags.KEY = [];

    // Process the affix buffer into a list of applicable lines.
    while (index > -1) {
      pushLine(aff.slice(last, index));
      last = index + 1;
      index = aff.indexOf('\n', last);
    }

    pushLine(aff.slice(last));

    // Process each line.
    index = -1;

    while (++index < lines.length) {
      line = lines[index];
      parts = line.split(whiteSpaceExpression$1);
      ruleType = parts[0];

      if (ruleType === 'REP') {
        count = index + parseInt(parts[1], 10);

        while (++index <= count) {
          parts = lines[index].split(whiteSpaceExpression$1);
          replacementTable.push([parts[1], parts[2]]);
        }

        index--;
      } else if (ruleType === 'ICONV' || ruleType === 'OCONV') {
        count = index + parseInt(parts[1], 10);
        entry = conversion[ruleType === 'ICONV' ? 'in' : 'out'];

        while (++index <= count) {
          parts = lines[index].split(whiteSpaceExpression$1);
          entry.push([new RegExp(parts[1], 'g'), parts[2]]);
        }

        index--;
      } else if (ruleType === 'COMPOUNDRULE') {
        count = index + parseInt(parts[1], 10);

        while (++index <= count) {
          rule = lines[index].split(whiteSpaceExpression$1)[1];
          position = -1;

          compoundRules.push(rule);

          while (++position < rule.length) {
            compoundRuleCodes[rule.charAt(position)] = [];
          }
        }

        index--;
      } else if (ruleType === 'PFX' || ruleType === 'SFX') {
        count = index + parseInt(parts[3], 10);

        rule = {
          type: ruleType,
          combineable: parts[2] === 'Y',
          entries: []
        };

        rules[parts[1]] = rule;

        while (++index <= count) {
          parts = lines[index].split(whiteSpaceExpression$1);
          remove = parts[2];
          add = parts[3].split('/');
          source = parts[4];

          entry = {
            add: '',
            remove: '',
            match: '',
            continuation: parse$2(flags, add[1])
          };

          if (add && add[0] !== '0') {
            entry.add = add[0];
          }

          try {
            if (remove !== '0') {
              entry.remove = ruleType === 'SFX' ? end(remove) : remove;
            }

            if (source && source !== '.') {
              entry.match = ruleType === 'SFX' ? end(source) : start(source);
            }
          } catch (_) {
            // Ignore invalid regex patterns.
            entry = null;
          }

          if (entry) {
            rule.entries.push(entry);
          }
        }

        index--;
      } else if (ruleType === 'TRY') {
        source = parts[1];
        offset = -1;
        value = [];

        while (++offset < source.length) {
          character = source.charAt(offset);

          if (character.toLowerCase() === character) {
            value.push(character);
          }
        }

        // Some dictionaries may forget a character.
        // Notably `en` forgets `j`, `x`, and `y`.
        offset = -1;

        while (++offset < alphabet.length) {
          if (source.indexOf(alphabet[offset]) < 0) {
            value.push(alphabet[offset]);
          }
        }

        flags[ruleType] = value;
      } else if (ruleType === 'KEY') {
        push$3.apply(flags[ruleType], parts[1].split('|'));
      } else if (ruleType === 'COMPOUNDMIN') {
        flags[ruleType] = Number(parts[1]);
      } else if (ruleType === 'ONLYINCOMPOUND') {
        // If we add this ONLYINCOMPOUND flag to `compoundRuleCodes`, then
        // `parseDic` will do the work of saving the list of words that are
        // compound-only.
        flags[ruleType] = parts[1];
        compoundRuleCodes[parts[1]] = [];
      } else if (
        ruleType === 'FLAG' ||
        ruleType === 'KEEPCASE' ||
        ruleType === 'NOSUGGEST' ||
        ruleType === 'WORDCHARS'
      ) {
        flags[ruleType] = parts[1];
      } else {
        // Default handling: set them for now.
        flags[ruleType] = parts[1];
      }
    }

    // Default for `COMPOUNDMIN` is `3`.
    // See `man 4 hunspell`.
    if (isNaN(flags.COMPOUNDMIN)) {
      flags.COMPOUNDMIN = 3;
    }

    if (!flags.KEY.length) {
      flags.KEY = defaultKeyboardLayout;
    }

    /* istanbul ignore if - Dictionaries seem to always have this. */
    if (!flags.TRY) {
      flags.TRY = alphabet.concat();
    }

    if (!flags.KEEPCASE) {
      flags.KEEPCASE = false;
    }

    return {
      compoundRuleCodes: compoundRuleCodes,
      replacementTable: replacementTable,
      conversion: conversion,
      compoundRules: compoundRules,
      rules: rules,
      flags: flags
    }

    function pushLine(line) {
      line = line.trim();

      // Hash can be a valid flag, so we only discard line that starts with it.
      if (line && line.charCodeAt(0) !== 35 /* `#` */) {
        lines.push(line);
      }
    }
  }

  // Wrap the `source` of an expression-like string so that it matches only at
  // the end of a value.
  function end(source) {
    return new RegExp(source + '$')
  }

  // Wrap the `source` of an expression-like string so that it matches only at
  // the start of a value.
  function start(source) {
    return new RegExp('^' + source)
  }

  var normalize_1 = normalize$2;

  // Normalize `value` with patterns.
  function normalize$2(value, patterns) {
    var index = -1;

    while (++index < patterns.length) {
      value = value.replace(patterns[index][0], patterns[index][1]);
    }

    return value
  }

  var flag_1 = flag$4;

  // Check whether a word has a flag.
  function flag$4(values, value, flags) {
    return flags && value in values && flags.indexOf(values[value]) > -1
  }

  var flag$3 = flag_1;

  var exact_1 = exact$2;

  // Check spelling of `value`, exactly.
  function exact$2(context, value) {
    var index = -1;

    if (context.data[value]) {
      return !flag$3(context.flags, 'ONLYINCOMPOUND', context.data[value])
    }

    // Check if this might be a compound word.
    if (value.length >= context.flags.COMPOUNDMIN) {
      while (++index < context.compoundRules.length) {
        if (context.compoundRules[index].test(value)) {
          return true
        }
      }
    }

    return false
  }

  var normalize$1 = normalize_1;
  var exact$1 = exact_1;
  var flag$2 = flag_1;

  var form_1 = form$3;

  // Find a known form of `value`.
  function form$3(context, value, all) {
    var normal = value.trim();
    var alternative;

    if (!normal) {
      return null
    }

    normal = normalize$1(normal, context.conversion.in);

    if (exact$1(context, normal)) {
      if (!all && flag$2(context.flags, 'FORBIDDENWORD', context.data[normal])) {
        return null
      }

      return normal
    }

    // Try sentence case if the value is uppercase.
    if (normal.toUpperCase() === normal) {
      alternative = normal.charAt(0) + normal.slice(1).toLowerCase();

      if (ignore(context.flags, context.data[alternative], all)) {
        return null
      }

      if (exact$1(context, alternative)) {
        return alternative
      }
    }

    // Try lowercase.
    alternative = normal.toLowerCase();

    if (alternative !== normal) {
      if (ignore(context.flags, context.data[alternative], all)) {
        return null
      }

      if (exact$1(context, alternative)) {
        return alternative
      }
    }

    return null
  }

  function ignore(flags, dict, all) {
    return (
      flag$2(flags, 'KEEPCASE', dict) || all || flag$2(flags, 'FORBIDDENWORD', dict)
    )
  }

  var form$2 = form_1;

  var correct_1 = correct;

  // Check spelling of `value`.
  function correct(value) {
    return Boolean(form$2(this, value))
  }

  var casing_1 = casing$1;

  // Get the casing of `value`.
  function casing$1(value) {
    var head = exact(value.charAt(0));
    var rest = value.slice(1);

    if (!rest) {
      return head
    }

    rest = exact(rest);

    if (head === rest) {
      return head
    }

    if (head === 'u' && rest === 'l') {
      return 's'
    }

    return null
  }

  function exact(value) {
    return value === value.toLowerCase()
      ? 'l'
      : value === value.toUpperCase()
      ? 'u'
      : null
  }

  var casing = casing_1;
  var normalize = normalize_1;
  var flag$1 = flag_1;
  var form$1 = form_1;

  var suggest_1 = suggest;

  var push$2 = [].push;

  // Suggest spelling for `value`.
  // eslint-disable-next-line complexity
  function suggest(value) {
    var self = this;
    var charAdded = {};
    var suggestions = [];
    var weighted = {};
    var memory;
    var replacement;
    var edits = [];
    var values;
    var index;
    var offset;
    var position;
    var count;
    var otherOffset;
    var otherCharacter;
    var character;
    var group;
    var before;
    var after;
    var upper;
    var insensitive;
    var firstLevel;
    var previous;
    var next;
    var nextCharacter;
    var max;
    var distance;
    var size;
    var normalized;
    var suggestion;
    var currentCase;

    value = normalize(value.trim(), self.conversion.in);

    if (!value || self.correct(value)) {
      return []
    }

    currentCase = casing(value);

    // Check the replacement table.
    index = -1;

    while (++index < self.replacementTable.length) {
      replacement = self.replacementTable[index];
      offset = value.indexOf(replacement[0]);

      while (offset > -1) {
        edits.push(value.replace(replacement[0], replacement[1]));
        offset = value.indexOf(replacement[0], offset + 1);
      }
    }

    // Check the keyboard.
    index = -1;

    while (++index < value.length) {
      character = value.charAt(index);
      before = value.slice(0, index);
      after = value.slice(index + 1);
      insensitive = character.toLowerCase();
      upper = insensitive !== character;
      charAdded = {};

      offset = -1;

      while (++offset < self.flags.KEY.length) {
        group = self.flags.KEY[offset];
        position = group.indexOf(insensitive);

        if (position < 0) {
          continue
        }

        otherOffset = -1;

        while (++otherOffset < group.length) {
          if (otherOffset !== position) {
            otherCharacter = group.charAt(otherOffset);

            if (charAdded[otherCharacter]) {
              continue
            }

            charAdded[otherCharacter] = true;

            if (upper) {
              otherCharacter = otherCharacter.toUpperCase();
            }

            edits.push(before + otherCharacter + after);
          }
        }
      }
    }

    // Check cases where one of a double character was forgotten, or one too many
    // were added, up to three “distances”.  This increases the success-rate by 2%
    // and speeds the process up by 13%.
    index = -1;
    nextCharacter = value.charAt(0);
    values = [''];
    max = 1;
    distance = 0;

    while (++index < value.length) {
      character = nextCharacter;
      nextCharacter = value.charAt(index + 1);
      before = value.slice(0, index);

      replacement = character === nextCharacter ? '' : character + character;
      offset = -1;
      count = values.length;

      while (++offset < count) {
        if (offset <= max) {
          values.push(values[offset] + replacement);
        }

        values[offset] += character;
      }

      if (++distance < 3) {
        max = values.length;
      }
    }

    push$2.apply(edits, values);

    // Ensure the capitalised and uppercase values are included.
    values = [value];
    replacement = value.toLowerCase();

    if (value === replacement || currentCase === null) {
      values.push(value.charAt(0).toUpperCase() + replacement.slice(1));
    }

    replacement = value.toUpperCase();

    if (value !== replacement) {
      values.push(replacement);
    }

    // Construct a memory object for `generate`.
    memory = {
      state: {},
      weighted: weighted,
      suggestions: suggestions
    };

    firstLevel = generate(self, memory, values, edits);

    // While there are no suggestions based on generated values with an
    // edit-distance of `1`, check the generated values, `SIZE` at a time.
    // Basically, we’re generating values with an edit-distance of `2`, but were
    // doing it in small batches because it’s such an expensive operation.
    previous = 0;
    max = Math.min(firstLevel.length, Math.pow(Math.max(15 - value.length, 3), 3));
    size = Math.max(Math.pow(10 - value.length, 3), 1);

    while (!suggestions.length && previous < max) {
      next = previous + size;
      generate(self, memory, firstLevel.slice(previous, next));
      previous = next;
    }

    // Sort the suggestions based on their weight.
    suggestions.sort(sort);

    // Normalize the output.
    values = [];
    normalized = [];
    index = -1;

    while (++index < suggestions.length) {
      suggestion = normalize(suggestions[index], self.conversion.out);
      replacement = suggestion.toLowerCase();

      if (normalized.indexOf(replacement) < 0) {
        values.push(suggestion);
        normalized.push(replacement);
      }
    }

    // BOOM! All done!
    return values

    function sort(a, b) {
      return sortWeight(a, b) || sortCasing(a, b) || sortAlpha(a, b)
    }

    function sortWeight(a, b) {
      return weighted[a] === weighted[b] ? 0 : weighted[a] > weighted[b] ? -1 : 1
    }

    function sortCasing(a, b) {
      var leftCasing = casing(a);
      var rightCasing = casing(b);

      return leftCasing === rightCasing
        ? 0
        : leftCasing === currentCase
        ? -1
        : rightCasing === currentCase
        ? 1
        : undefined
    }

    function sortAlpha(a, b) {
      return a.localeCompare(b)
    }
  }

  // Get a list of values close in edit distance to `words`.
  function generate(context, memory, words, edits) {
    var characters = context.flags.TRY;
    var data = context.data;
    var flags = context.flags;
    var result = [];
    var index = -1;
    var word;
    var before;
    var character;
    var nextCharacter;
    var nextAfter;
    var nextNextAfter;
    var nextUpper;
    var currentCase;
    var position;
    var after;
    var upper;
    var inject;
    var offset;

    // Check the pre-generated edits.
    if (edits) {
      while (++index < edits.length) {
        check(edits[index], true);
      }
    }

    // Iterate over given word.
    index = -1;

    while (++index < words.length) {
      word = words[index];
      before = '';
      character = '';
      nextCharacter = word.charAt(0);
      nextAfter = word;
      nextNextAfter = word.slice(1);
      nextUpper = nextCharacter.toLowerCase() !== nextCharacter;
      currentCase = casing(word);
      position = -1;

      // Iterate over every character (including the end).
      while (++position <= word.length) {
        before += character;
        after = nextAfter;
        nextAfter = nextNextAfter;
        nextNextAfter = nextAfter.slice(1);
        character = nextCharacter;
        nextCharacter = word.charAt(position + 1);
        upper = nextUpper;

        if (nextCharacter) {
          nextUpper = nextCharacter.toLowerCase() !== nextCharacter;
        }

        if (nextAfter && upper !== nextUpper) {
          // Remove.
          check(before + switchCase(nextAfter));

          // Switch.
          check(
            before +
              switchCase(nextCharacter) +
              switchCase(character) +
              nextNextAfter
          );
        }

        // Remove.
        check(before + nextAfter);

        // Switch.
        if (nextAfter) {
          check(before + nextCharacter + character + nextNextAfter);
        }

        // Iterate over all possible letters.
        offset = -1;

        while (++offset < characters.length) {
          inject = characters[offset];

          // Try uppercase if the original character was uppercased.
          if (upper && inject !== inject.toUpperCase()) {
            if (currentCase !== 's') {
              check(before + inject + after);
              check(before + inject + nextAfter);
            }

            inject = inject.toUpperCase();

            check(before + inject + after);
            check(before + inject + nextAfter);
          } else {
            // Add and replace.
            check(before + inject + after);
            check(before + inject + nextAfter);
          }
        }
      }
    }

    // Return the list of generated words.
    return result

    // Check and handle a generated value.
    function check(value, double) {
      var state = memory.state[value];
      var corrected;

      if (state !== Boolean(state)) {
        result.push(value);

        corrected = form$1(context, value);
        state = corrected && !flag$1(flags, 'NOSUGGEST', data[corrected]);

        memory.state[value] = state;

        if (state) {
          memory.weighted[value] = double ? 10 : 0;
          memory.suggestions.push(value);
        }
      }

      if (state) {
        memory.weighted[value]++;
      }
    }

    function switchCase(fragment) {
      var first = fragment.charAt(0);

      return (
        (first.toLowerCase() === first
          ? first.toUpperCase()
          : first.toLowerCase()) + fragment.slice(1)
      )
    }
  }

  var form = form_1;
  var flag = flag_1;

  var spell_1 = spell;

  // Check spelling of `word`.
  function spell(word) {
    var self = this;
    var value = form(self, word, true);

    // Hunspell also provides `root` (root word of the input word), and `compound`
    // (whether `word` was compound).
    return {
      correct: self.correct(word),
      forbidden: Boolean(
        value && flag(self.flags, 'FORBIDDENWORD', self.data[value])
      ),
      warn: Boolean(value && flag(self.flags, 'WARN', self.data[value]))
    }
  }

  var apply_1 = apply$1;

  // Apply a rule.
  function apply$1(value, rule, rules, words) {
    var index = -1;
    var entry;
    var next;
    var continuationRule;
    var continuation;
    var position;

    while (++index < rule.entries.length) {
      entry = rule.entries[index];
      continuation = entry.continuation;
      position = -1;

      if (!entry.match || entry.match.test(value)) {
        next = entry.remove ? value.replace(entry.remove, '') : value;
        next = rule.type === 'SFX' ? next + entry.add : entry.add + next;
        words.push(next);

        if (continuation && continuation.length) {
          while (++position < continuation.length) {
            continuationRule = rules[continuation[position]];

            if (continuationRule) {
              apply$1(next, continuationRule, rules, words);
            }
          }
        }
      }
    }

    return words
  }

  var apply = apply_1;

  var add_1$1 = add$4;

  var push$1 = [].push;

  var NO_RULES = [];

  // Add `rules` for `word` to the table.
  function addRules(dict, word, rules) {
    var curr = dict[word];

    // Some dictionaries will list the same word multiple times with different
    // rule sets.
    if (word in dict) {
      if (curr === NO_RULES) {
        dict[word] = rules.concat();
      } else {
        push$1.apply(curr, rules);
      }
    } else {
      dict[word] = rules.concat();
    }
  }

  function add$4(dict, word, codes, options) {
    var position = -1;
    var rule;
    var offset;
    var subposition;
    var suboffset;
    var combined;
    var newWords;
    var otherNewWords;

    // Compound words.
    if (
      !('NEEDAFFIX' in options.flags) ||
      codes.indexOf(options.flags.NEEDAFFIX) < 0
    ) {
      addRules(dict, word, codes);
    }

    while (++position < codes.length) {
      rule = options.rules[codes[position]];

      if (codes[position] in options.compoundRuleCodes) {
        options.compoundRuleCodes[codes[position]].push(word);
      }

      if (rule) {
        newWords = apply(word, rule, options.rules, []);
        offset = -1;

        while (++offset < newWords.length) {
          if (!(newWords[offset] in dict)) {
            dict[newWords[offset]] = NO_RULES;
          }

          if (rule.combineable) {
            subposition = position;

            while (++subposition < codes.length) {
              combined = options.rules[codes[subposition]];

              if (
                combined &&
                combined.combineable &&
                rule.type !== combined.type
              ) {
                otherNewWords = apply(
                  newWords[offset],
                  combined,
                  options.rules,
                  []
                );
                suboffset = -1;

                while (++suboffset < otherNewWords.length) {
                  if (!(otherNewWords[suboffset] in dict)) {
                    dict[otherNewWords[suboffset]] = NO_RULES;
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  var push = add_1$1;

  var add_1 = add$3;

  var NO_CODES = [];

  // Add `value` to the checker.
  function add$3(value, model) {
    var self = this;

    push(self.data, value, self.data[model] || NO_CODES, self);

    return self
  }

  var remove_1 = remove;

  // Remove `value` from the checker.
  function remove(value) {
    var self = this;

    delete self.data[value];

    return self
  }

  var wordCharacters_1 = wordCharacters;

  // Get the word characters defined in affix.
  function wordCharacters() {
    return this.flags.WORDCHARS || null
  }

  var parseCodes = ruleCodes_1;
  var add$2 = add_1$1;

  var dictionary$1 = parse$1;

  // Expressions.
  var whiteSpaceExpression = /\s/g;

  // Parse a dictionary.
  function parse$1(buf, options, dict) {
    // Parse as lines (ignoring the first line).
    var value = buf.toString('utf8');
    var last = value.indexOf('\n') + 1;
    var index = value.indexOf('\n', last);

    while (index > -1) {
      // Some dictionaries use tabs as comments.
      if (value.charCodeAt(last) !== 9 /* `\t` */) {
        parseLine(value.slice(last, index), options, dict);
      }

      last = index + 1;
      index = value.indexOf('\n', last);
    }

    parseLine(value.slice(last), options, dict);
  }

  // Parse a line in dictionary.
  function parseLine(line, options, dict) {
    var slashOffset = line.indexOf('/');
    var hashOffset = line.indexOf('#');
    var codes = '';
    var word;
    var result;

    // Find offsets.
    while (
      slashOffset > -1 &&
      line.charCodeAt(slashOffset - 1) === 92 /* `\` */
    ) {
      line = line.slice(0, slashOffset - 1) + line.slice(slashOffset);
      slashOffset = line.indexOf('/', slashOffset);
    }

    // Handle hash and slash offsets.
    // Note that hash can be a valid flag, so we should not just discard
    // everything after it.
    if (hashOffset > -1) {
      if (slashOffset > -1 && slashOffset < hashOffset) {
        word = line.slice(0, slashOffset);
        whiteSpaceExpression.lastIndex = slashOffset + 1;
        result = whiteSpaceExpression.exec(line);
        codes = line.slice(slashOffset + 1, result ? result.index : undefined);
      } else {
        word = line.slice(0, hashOffset);
      }
    } else if (slashOffset > -1) {
      word = line.slice(0, slashOffset);
      codes = line.slice(slashOffset + 1);
    } else {
      word = line;
    }

    word = word.trim();

    if (word) {
      add$2(dict, word, parseCodes(options.flags, codes.trim()), options);
    }
  }

  var parse = dictionary$1;

  var dictionary = add$1;

  // Add a dictionary file.
  function add$1(buf) {
    var self = this;
    var index = -1;
    var rule;
    var source;
    var character;
    var offset;

    parse(buf, self, self.data);

    // Regenerate compound expressions.
    while (++index < self.compoundRules.length) {
      rule = self.compoundRules[index];
      source = '';
      offset = -1;

      while (++offset < rule.length) {
        character = rule.charAt(offset);
        source += self.compoundRuleCodes[character].length
          ? '(?:' + self.compoundRuleCodes[character].join('|') + ')'
          : character;
      }

      self.compoundRules[index] = new RegExp(source, 'i');
    }

    return self
  }

  var personal = add;

  // Add a dictionary.
  function add(buf) {
    var self = this;
    var lines = buf.toString('utf8').split('\n');
    var index = -1;
    var line;
    var forbidden;
    var word;
    var flag;

    // Ensure there’s a key for `FORBIDDENWORD`: `false` cannot be set through an
    // affix file so its safe to use as a magic constant.
    if (self.flags.FORBIDDENWORD === undefined) self.flags.FORBIDDENWORD = false;
    flag = self.flags.FORBIDDENWORD;

    while (++index < lines.length) {
      line = lines[index].trim();

      if (!line) {
        continue
      }

      line = line.split('/');
      word = line[0];
      forbidden = word.charAt(0) === '*';

      if (forbidden) {
        word = word.slice(1);
      }

      self.add(word, line[1]);

      if (forbidden) {
        self.data[word].push(flag);
      }
    }

    return self
  }

  var buffer = isBuffer;
  var affix = affix_1;

  var lib = NSpell;

  var proto = NSpell.prototype;

  proto.correct = correct_1;
  proto.suggest = suggest_1;
  proto.spell = spell_1;
  proto.add = add_1;
  proto.remove = remove_1;
  proto.wordCharacters = wordCharacters_1;
  proto.dictionary = dictionary;
  proto.personal = personal;

  // Construct a new spelling context.
  function NSpell(aff, dic) {
    var index = -1;
    var dictionaries;

    if (!(this instanceof NSpell)) {
      return new NSpell(aff, dic)
    }

    if (typeof aff === 'string' || buffer(aff)) {
      if (typeof dic === 'string' || buffer(dic)) {
        dictionaries = [{dic: dic}];
      }
    } else if (aff) {
      if ('length' in aff) {
        dictionaries = aff;
        aff = aff[0] && aff[0].aff;
      } else {
        if (aff.dic) {
          dictionaries = [aff];
        }

        aff = aff.aff;
      }
    }

    if (!aff) {
      throw new Error('Missing `aff` in dictionary')
    }

    aff = affix(aff);

    this.data = Object.create(null);
    this.compoundRuleCodes = aff.compoundRuleCodes;
    this.replacementTable = aff.replacementTable;
    this.conversion = aff.conversion;
    this.compoundRules = aff.compoundRules;
    this.rules = aff.rules;
    this.flags = aff.flags;

    if (dictionaries) {
      while (++index < dictionaries.length) {
        if (dictionaries[index].dic) {
          this.dictionary(dictionaries[index].dic);
        }
      }
    }
  }

  return lib;

}));
