import nspell from "nspell";
import add from "./add";
import correct from "./correct";
import dictionary from "./dictionary";
import getDictionaries from "./get-dictionaries";
import getInnerDictionaries from "./get-inner-dictionaries";
import getInnerNameMap from "./get-inner-name-map";
import personal from "./personal";
import remove from "./remove";
import suggest from "./suggest";
import { BuiltInDictionary } from "./types";
var defaultDictionaries = Object.freeze({
    frontEnd: BuiltInDictionary.frontEnd,
});
var NameSpellChecker = /** @class */ (function () {
    function NameSpellChecker(dictionaries) {
        if (!dictionaries) {
            this.dictionaries = getDictionaries(Object.values(NameSpellChecker.defaultDictionaries));
        }
        else {
            this.dictionaries = getDictionaries(dictionaries);
        }
        this.innerDictionaries = getInnerDictionaries(this.dictionaries);
        this.innerNameMap = getInnerNameMap(this);
        this.nspellInstance = nspell(this.innerDictionaries);
    }
    Object.defineProperty(NameSpellChecker, "defaultDictionaries", {
        get: function () {
            return defaultDictionaries;
        },
        enumerable: false,
        configurable: true
    });
    NameSpellChecker.prototype.suggest = function (str) {
        return suggest(this, str);
    };
    NameSpellChecker.prototype.correct = function (str) {
        return correct(this, str);
    };
    NameSpellChecker.prototype.add = function (brandName) {
        add(this, brandName);
        return this;
    };
    NameSpellChecker.prototype.remove = function (brandName) {
        remove(this, brandName);
        return this;
    };
    NameSpellChecker.prototype.dictionary = function (dic) {
        dictionary(this, dic);
        return this;
    };
    NameSpellChecker.prototype.personal = function (personalDic) {
        personal(this, personalDic);
        return this;
    };
    return NameSpellChecker;
}());
export default NameSpellChecker;
